import { ApiCall } from "tsrpc";
import { ReqFight, ResFight } from "../../../shared/protocols/event/huangqijiuguan/PtlFight";
import { HuoDongFun } from "../../../public/huodongfun";
import HQJGFun from "./fun";
import { fightResult } from "../../../shared/fightControl/fightType";
import { FightFun } from "../../../public/fight";

export default async function (call: ApiCall<ReqFight, ResFight>) {
    let hd = await HQJGFun.HdInfo(call, call.req.hdid);

    if (!hd) {
        // 无此活动
        return call.error('', { code: -1, message: globalThis.lng.huodong_open_1 })
    }

    // 个人活动信息
    let mydata = await HQJGFun.getMydata(call, call.req.hdid);

    // 开始战斗
    let result: fightResult = await FightFun.fightNpc(call, hd.data.bossId, 'hqjg');

    let up = false;
    let today = HQJGFun.today(hd);
    // 历史最大伤害
    if (result.totalDamage[0] > mydata.bossres.maxdps) {
        up = true;
        mydata.bossres.maxdps = result.totalDamage[0];
    }

    // 今日最大伤害
    if (result.totalDamage[0] > (mydata.bossres.todaydps[today] || 0)) {
        up = true;
        mydata.bossres.todaydps[today] = result.totalDamage[0];
    }

    // 更新伤害数据
    up && await HQJGFun.setMyData(call.uid, call.req.hdid, { bossres: mydata.bossres });

    call.succ({ data: mydata, result: result });
}